@@ -54,6 +54,11 @@
 #' @param log.degree a character string which contains \code{"x"} if the (horizontal) degree axis in the plot
 #' of the estimated visibilites for each respondent verses their reported network sizes be logarithmic. 
 #' A value of \code{"y"} uses a logarithmic visibility axis and \code{"xy"} both. The default is \code{""}, no logarithmic axes.
+#' @param layout a vector of the form ‘c(nv, nc)’. The produced plots, in particular the MCMC diagnostics, will be draw in 
+#' figures in ‘nv’-by-‘nc’ arrays per page. `nc` is typically 2 to have the trace plots on the left and density plots on the right.
+#' `nv` is then the number of variables per page, by default 3.
+#' @param method character; The method to use for density estimation (default Gaussian Kernel; "bgk").
+#' "Bayes" uses a Bayesian density estimator which has good properties.
 #' @param \dots further arguments passed to or from other methods.
 #' @seealso The model fitting function \code{\link{posteriorsize}},
 #' \code{\link[graphics]{plot}}.
@@ -72,11 +77,11 @@
 #' 
 #' Gile, Krista J. and Handcock, Mark S. (2014) \pkg{sspse}: Estimating Hidden 
 #' Population Size using Respondent Driven Sampling Data
-#' R package, Los Angeles, CA.  Version 0.5, \url{http://hpmrg.org}.
+#' R package, Los Angeles, CA.  Version 0.5, \url{https://github.com/HPMRG}.
 #' 
 #' Handcock MS (2003).  \pkg{degreenet}: Models for Skewed Count Distributions
 #' Relevant to Networks.  Statnet Project, Seattle, WA.  Version 1.2,
-#' \url{http://statnetproject.org}.
+#' \url{https://statnet.org}.
 #' 
 #' Handcock, Mark S., Gile, Krista J. and Mar, Corinne M. (2014)
 #' \emph{Estimating Hidden Population Size using Respondent-Driven Sampling
@@ -91,7 +96,7 @@
 #' # Here interval=1 so that it will run faster. It should be higher in a 
 #' # real application.
 #' fit <- posteriorsize(fauxmadrona, median.prior.size=1000,
-#'                                  burnin=100, interval=1, samplesize=100)
+#'                                  burnin=20, interval=1, samplesize=100)
 #' summary(fit)
 #' # Let's look at some MCMC diagnostics
 #' plot(fit, mcmc=TRUE)
@@ -101,11 +106,13 @@
 #' @export
 plot.sspse <- function(x,
 		       xlim=NULL,support=1000,HPD.level=0.90,N=NULL,ylim=NULL,mcmc=FALSE,type="all",
-		       main="Posterior for population size",smooth=4,include.tree=TRUE,cex.main=1,log.degree="",...){
+		       main="Posterior for population size",smooth=4,include.tree=TRUE,cex.main=1,log.degree="",
+                       layout=c(3,2),method="bgk",...){
   p.args <- as.list( sys.call() )[-c(1,2)]
   formal.args<-formals(sys.function())[-c(1)]
 
-  control<-list()
+ #control<-list()
+  control<-list(samples=4000, burnin=1000)
   names.formal.args <- names(formal.args)
   names.formal.args <- names.formal.args[-match("...",names.formal.args)]
   for(arg in names.formal.args){ control[arg]<-list(get(arg)) }
@@ -120,12 +127,15 @@ attr(out,"mcpar") <- attr(x$sample,"mcpar")
 attr(out,"class") <- attr(x$sample,"class")
 # sabline <- function(v,...){graphics::segments(x0=v,...,y0=control$ylim[1],y1=control$ylim[2])}
 sabline <- function(v,...){graphics::segments(x0=v,...,y0=graphics::par("usr")[3],y1=graphics::par("usr")[4])}
+oldpar <- par("mfrow")
+par(mfrow=layout)
 if(!is.null(out) & control$type == "mcmc"){
   mcmc.len <- min(1000, nrow(out))
   a=round(seq.int(from=1,to=nrow(out),length.out=mcmc.len))
   mcp <- attr(out,"mcpar")
   b=coda::mcmc(out[a,],start=mcp[1],end=mcp[2],thin=floor((mcp[2]-mcp[1])/mcmc.len + 1))
-  graphics::plot(b)
+  graphics::plot(b, auto.layout = FALSE)
+  par(mfrow=oldpar)
   return(invisible())
 }
 #suppressMessages(require(locfit,quietly=TRUE))
@@ -143,18 +153,31 @@ if(!is.null(out)){
   outN <- out[,"N"]
   ##a=locfit( ~ lp(outN, nn=0.35, h=0, maxk=500))
   xp <- seq(x$n,x$maxN, length=control$support)
-# posdensN=bgk_kde(data=outN,n=2^(ceiling(log(x$maxN-x$n)/log(2))),MIN=x$n,MAX=x$maxN, smooth=smooth)
-# maxposdensN <- max(posdensN[1,],na.rm=TRUE)
-# posdensN <- stats::spline(x=posdensN[1,],y=posdensN[2,],xout=xp)$y
-  posdensN=KernSmooth::bkde(x=log(outN), kernel = "normal", gridsize = length(xp), range.x=log(c(x$n,x$maxN)))
-# posdensN=density(x=log(outN),n=control$support,from=log(x$n),to=log(x$maxN))
-# xp=exp(posdensN$x)
-# posdensN=posdensN$y/xp
-  maxposdensN <- max(exp(posdensN$x),na.rm=TRUE)
-  posdensN <- stats::spline(x=exp(posdensN$x),y=posdensN$y/exp(posdensN$x),xout=xp)$y
-  posdensN[xp > maxposdensN] <- 0
-  #a=locfit::locfit( ~ lp(outN,nn=0.5))
-  #posdensN <- predict(a, newdata=xp)
+  if(method=="bgk" || !requireNamespace("densEstBayes", quietly = TRUE)){
+#   posdensN=bgk_kde(data=outN,n=2^(ceiling(log(x$maxN-x$n)/log(2))),MIN=x$n,MAX=x$maxN, smooth=smooth)
+#   maxposdensN <- max(posdensN[1,],na.rm=TRUE)
+#   posdensN <- stats::spline(x=posdensN[1,],y=posdensN[2,],xout=xp)$y
+    posdensN=KernSmooth::bkde(x=log(outN), kernel = "normal", gridsize = length(xp), range.x=log(c(x$n,x$maxN)))
+#   posdensN=density(x=log(outN),n=control$support,from=log(x$n),to=log(x$maxN))
+#   xp=exp(posdensN$x)
+#   posdensN=posdensN$y/xp
+    maxposdensN <- max(exp(posdensN$x),na.rm=TRUE)
+    posdensN <- stats::spline(x=exp(posdensN$x),y=posdensN$y/exp(posdensN$x),xout=xp)$y
+    posdensN[xp > maxposdensN] <- 0
+    #a=locfit::locfit( ~ lp(outN,nn=0.5))
+    #posdensN <- predict(a, newdata=xp)
+  }else{
+    a=densEstBayes::densEstBayes(outN,method="NUTS",
+      control=densEstBayes::densEstBayes.control(range.x=c(x$n*0.95,x$maxN*1.05),#numBins=min(401,round(length(outN)/2)),
+                                                 nKept=control$samples,nWarm=control$burnin))
+    xTrang <- seq(-0.05, 1.05, length = length(xp))
+    Xg <- cbind(1,xTrang)
+    Zg <- .ZOSull(xTrang,intKnots=a$intKnots,range.x=c(-0.05,1.05))
+    Cg <- cbind(Xg,Zg)
+    betauMCMC <- a$stochaFitObj$betauMCMC
+    etaHatMCMC <- crossprod(t(Cg),betauMCMC)
+    posdensN <- exp(apply(etaHatMCMC, 1, mean))
+  }
   posdensN <- control$support*posdensN / ((x$maxN-x$n)*sum(posdensN))
   #
   if(is.null(control$xlim)){control$xlim <- stats::quantile(outN,0.99)}
@@ -333,5 +356,6 @@ if(control$type %in% c("prior","all")){
 # round(mp),round(l50),round(map),round(l90),round(hpd[1]),round(hpd[2])))
 #
 }
+par(mfrow=oldpar)
 invisible()
 }
