@@ -1,15 +1,15 @@
 poscmp<-function(s,s2=NULL,rc=rep(FALSE,length=length(s2)),maxN=NULL,
-                  K=2*max(c(s,s2)), nk=NULL, n=length(s), n2=length(s2),
-                  mean.prior.degree=7, sd.prior.degree=3,
-                  df.mean.prior=1, df.sd.prior=5,
+                  K=2*max(c(s,s2)), n=length(s), n2=length(s2),
+                  mean.prior.visibility=7, sd.prior.visibility=3,
+                  df.mean.prior.visibility=1, df.sd.prior.visibility=5,
                   beta0.mean.prior=-3, beta1.mean.prior=0,
                   beta0.sd.prior=10, beta1.sd.prior=10,
                   mem.optimism.prior=1, df.mem.optimism.prior=5, 
-                  mem.sd.prior=5, df.mem.sd.prior=3, 
+                  mem.scale.prior=1, df.mem.scale.prior=3, 
                   muproposal=0.1, 
-                  sigmaproposal=0.15, 
+                  nuproposal=0.15, 
                   beta0proposal=0.1, beta1proposal=0.001,
-                  memmuproposal=0.1, memsdproposal=0.15,
+                  memmuproposal=0.1, memscaleproposal=0.15,
                   visibility=TRUE,
                   Np=0,
                   samplesize=10,burnin=0,interval=1,burnintheta=500,burninbeta=20,
@@ -27,6 +27,8 @@ poscmp<-function(s,s2=NULL,rc=rep(FALSE,length=length(s2)),maxN=NULL,
                   supplied=list(maxN=maxN),
               num.recruits=NULL,
               recruit.times=NULL,
+              num.recruits2=NULL,
+              recruit.times2=NULL,
               max.coupons=NULL,
                   verbose=TRUE){
     #this function takes a vector of population sizes and a vector s of 
@@ -34,19 +36,16 @@ poscmp<-function(s,s2=NULL,rc=rep(FALSE,length=length(s2)),maxN=NULL,
     #s values must all be positive integers
     if(!is.null(seed))  set.seed(as.integer(seed))
     #
-    # Cap the maximum degree to K
+    # Cap the maximum visibility to K
     #
-    s[s>K] <- K
-    if(is.null(nk)){nk=tabulate(s,nbins=K)}
-    if(!is.null(s2)) s2[s2>K] <- K
+    # s[s>K] <- K
     #
     # Transform observed mean parametrization to log-normal
     # parametrization
     #
-    out <- cmp.natural(mu=mean.prior.degree, sigma=sd.prior.degree)
-    mu <- log(out$lambda)
-    sigma <- out$nu
-#   sigma <- max(0.00001, sigma)
+    out <- cmp.to.natural(mu=mean.prior.visibility, sigma=sd.prior.visibility)
+    lnlam <- log(out$lambda)
+    nu <- out$nu
     #
     if(visibility){
       dimsample <- 5+Np+4
@@ -55,8 +54,8 @@ poscmp<-function(s,s2=NULL,rc=rep(FALSE,length=length(s2)),maxN=NULL,
     }
     #
     # Determine if we are in the two-sample case or the one-sample
+    n1 = n
     if(!is.null(s2)){
-     n1 = n
      n0 = sum(rc)
      n = n1 + n2 - n0 # The number of unique people seen
     }
@@ -79,6 +78,57 @@ poscmp<-function(s,s2=NULL,rc=rep(FALSE,length=length(s2)),maxN=NULL,
                   supplied=supplied,
                   verbose=verbose)
     if(!is.null(s2)){
+     if(visibility){
+      cat(sprintf("Using Capture-recapture measurement error model with K = %d.\n",K))
+      nk=tabulate(c(s,s2[!rc]),nbins=K)
+      Cret <- .C("gcmpvis2",
+              pop12=as.integer(c(s, s2[!rc], rep(0,prior$maxN-length(s)-length(s2[!rc])))),
+              pop21=as.integer(c(s2,sum(s)-sum(s2[rc]), rep(0,prior$maxN-length(s2)-1))),
+              nk=as.integer(nk),
+              K=as.integer(K),
+              n1=as.integer(n1),
+              n2=as.integer(n2),
+              n0=as.integer(n0),
+              samplesize=as.integer(samplesize),
+              burnin=as.integer(burnin),
+              interval=as.integer(interval),
+              mu=as.double(mean.prior.visibility), df.mean.prior.visibility=as.double(df.mean.prior.visibility),
+              sigma=as.double(sd.prior.visibility), df.sd.prior.visibility=as.double(df.sd.prior.visibility),
+              lnlam=as.double(lnlam), nu=as.double(nu),
+              beta0.mean.prior=as.double(beta0.mean.prior), beta0.sd.prior=as.double(beta0.sd.prior),
+              beta1.mean.prior=as.double(beta1.mean.prior), beta1.sd.prior=as.double(beta1.sd.prior),
+              mem.optimism.prior=as.double(log(mem.optimism.prior)), df.mem.optimism.prior=as.double(df.mem.optimism.prior),
+              mem.scale.prior=as.double(mem.scale.prior), df.mem.scale.prior=as.double(df.mem.scale.prior),
+              Np=as.integer(Np),
+              srd=as.integer(s),
+              numrec=as.integer(num.recruits),
+              rectime=as.double(recruit.times),
+              srd2=as.integer(s2),
+              numrec2=as.integer(num.recruits2),
+              rectime2=as.double(recruit.times2),
+              rc=as.integer(rc),
+              maxcoupons=as.integer(max.coupons),
+              muproposal=as.double(muproposal),
+              nuproposal=as.double(nuproposal),
+              beta0proposal=as.double(beta0proposal), beta1proposal=as.double(beta1proposal),
+              memmuproposal=as.double(memmuproposal), memscaleproposal=as.double(memscaleproposal),
+              N=as.integer(prior$N),
+              maxN=as.integer(prior$maxN),
+              sample=double(samplesize*dimsample),
+              vsample=integer(samplesize*n1),
+              vsample2=integer(samplesize*n2),
+              ppos=double(K),
+              lpriorm=as.double(prior$lprior),
+              burnintheta=as.integer(burnintheta),
+              burninbeta=as.integer(burninbeta),
+              verbose=as.integer(verbose), PACKAGE="sspse")
+     }else{
+      cat(sprintf("Using Capture-recapture non-measurement error model with K = %d.\n",K))
+       s[ s>K] <- K
+      s2[s2>K] <- K
+       s[ s<1] <- 1
+      s2[s2<1] <- 1
+      nk=tabulate(c(s,s2[!rc]),nbins=K)
       Cret <- .C("gcmp2",
               pop12=as.integer(c(s, s2[!rc], rep(0,prior$maxN-length(s)-length(s2[!rc])))),
               pop21=as.integer(c(s2,sum(s)-sum(s2[rc]), rep(0,prior$maxN-length(s2)-1))),
@@ -90,67 +140,73 @@ poscmp<-function(s,s2=NULL,rc=rep(FALSE,length=length(s2)),maxN=NULL,
               samplesize=as.integer(samplesize),
               burnin=as.integer(burnin),
               interval=as.integer(interval),
-              mu=as.double(mu), df.mean.prior=as.double(df.mean.prior),
-              sigma=as.double(sigma), df.sd.prior=as.double(df.sd.prior),
+              mu=as.double(mean.prior.visibility), df.mean.prior.visibility=as.double(df.mean.prior.visibility),
+              sigma=as.double(sd.prior.visibility), df.sd.prior.visibility=as.double(df.sd.prior.visibility),
+              lnlam=as.double(lnlam), nu=as.double(nu),
               Np=as.integer(Np),
               muproposal=as.double(muproposal),
-              sigmaproposal=as.double(sigmaproposal),
+              nuproposal=as.double(nuproposal),
               N=as.integer(prior$N),
               maxN=as.integer(prior$maxN),
               sample=double(samplesize*dimsample),
               ppos=double(K),
               lpriorm=as.double(prior$lprior),
               burnintheta=as.integer(burnintheta),
-              burninbeta=as.integer(burninbeta),
               verbose=as.integer(verbose), PACKAGE="sspse")
+     }
     }else{
      if(visibility){
+      cat(sprintf("Using CMP measurement error model with K = %d.\n",K))
       Cret <- .C("gcmpvis",
-              pop=as.integer(c(s,rep(0,prior$maxN-n))),
-              nk=as.integer(nk),
+              pop=as.integer(c(s,rep(0,prior$maxN-n1))),
               K=as.integer(K),
-              n=as.integer(n),
+              n=as.integer(n1),
               samplesize=as.integer(samplesize),
               burnin=as.integer(burnin),
               interval=as.integer(interval),
-              mu=as.double(mean.prior.degree), df.mean.prior=as.double(df.mean.prior),
-              sigma=as.double(sd.prior.degree), df.sd.prior=as.double(df.sd.prior),
+              mu=as.double(mean.prior.visibility), df.mean.prior.visibility=as.double(df.mean.prior.visibility),
+              sigma=as.double(sd.prior.visibility), df.sd.prior.visibility=as.double(df.sd.prior.visibility),
+              lnlam=as.double(lnlam), nu=as.double(nu),
               beta0.mean.prior=as.double(beta0.mean.prior), beta0.sd.prior=as.double(beta0.sd.prior),
               beta1.mean.prior=as.double(beta1.mean.prior), beta1.sd.prior=as.double(beta1.sd.prior),
               mem.optimism.prior=as.double(log(mem.optimism.prior)), df.mem.optimism.prior=as.double(df.mem.optimism.prior),
-              mem.sd.prior=as.double(mem.sd.prior), df.mem.sd.prior=as.double(df.mem.sd.prior),
+              mem.scale.prior=as.double(mem.scale.prior), df.mem.scale.prior=as.double(df.mem.scale.prior),
               Np=as.integer(Np),
               srd=as.integer(s),
-              numrec=as.double(num.recruits),
+              numrec=as.integer(num.recruits),
               rectime=as.double(recruit.times),
               maxcoupons=as.integer(max.coupons),
               muproposal=as.double(muproposal),
-              sigmaproposal=as.double(sigmaproposal),
+              nuproposal=as.double(nuproposal),
               beta0proposal=as.double(beta0proposal), beta1proposal=as.double(beta1proposal),
-              memmuproposal=as.double(memmuproposal), memsdproposal=as.double(memsdproposal),
+              memmuproposal=as.double(memmuproposal), memscaleproposal=as.double(memscaleproposal),
               N=as.integer(prior$N),
               maxN=as.integer(prior$maxN),
               sample=double(samplesize*dimsample),
-              vsample=integer(samplesize*n),
-              ppos=double(K),
+              vsample=integer(samplesize*n1),
+              posu=double(K),
+              posd=double(K),
               lpriorm=as.double(prior$lprior),
               burnintheta=as.integer(burnintheta),
               burninbeta=as.integer(burninbeta),
               verbose=as.integer(verbose), PACKAGE="sspse")
      }else{
+      cat(sprintf("Using non-measurement error model with K = %d.\n",K))
+      s[s>K] <- K
+      s[s<1] <- 1
       Cret <- .C("gcmp",
-              pop=as.integer(c(s,rep(0,prior$maxN-n))),
-              nk=as.integer(nk),
+              pop=as.integer(c(s,rep(0,prior$maxN-n1))),
               K=as.integer(K),
-              n=as.integer(n),
+              n=as.integer(n1),
               samplesize=as.integer(samplesize),
               burnin=as.integer(burnin),
               interval=as.integer(interval),
-              mu=as.double(mean.prior.degree), df.mean.prior=as.double(df.mean.prior),
-              sigma=as.double(sd.prior.degree), df.sd.prior=as.double(df.sd.prior),
+              mu=as.double(mean.prior.visibility), df.mean.prior.visibility=as.double(df.mean.prior.visibility),
+              sigma=as.double(sd.prior.visibility), df.sd.prior.visibility=as.double(df.sd.prior.visibility),
+              lnlam=as.double(lnlam), nu=as.double(nu),
               Np=as.integer(Np),
               muproposal=as.double(muproposal),
-              sigmaproposal=as.double(sigmaproposal),
+              nuproposal=as.double(nuproposal),
               N=as.integer(prior$N),
               maxN=as.integer(prior$maxN),
               sample=double(samplesize*dimsample),
@@ -164,9 +220,13 @@ poscmp<-function(s,s2=NULL,rc=rep(FALSE,length=length(s2)),maxN=NULL,
     degnames <- NULL
     if(Np>0){degnames <- c(degnames,paste("pdeg",1:Np,sep=""))}
     if(visibility){
-     colnamessample <- c("N","mu","sigma","degree1","totalsize","beta0","beta1","mem.optimism","mem.sd")
-     Cret$vsample<-matrix(Cret$vsample,nrow=samplesize,ncol=n,byrow=TRUE)
-     colnames(Cret$vsample) <- 1:n
+     colnamessample <- c("N","mu","sigma","visibility1","totalsize","beta0","beta1","mem.optimism","mem.scale")
+     Cret$vsample<-matrix(Cret$vsample,nrow=samplesize,ncol=n1,byrow=TRUE)
+     colnames(Cret$vsample) <- 1:n1
+     if(!is.null(s2)){
+      Cret$vsample2<-matrix(Cret$vsample2,nrow=samplesize,ncol=n2,byrow=TRUE)
+      colnames(Cret$vsample2) <- 1:n2
+     }
      max.mu <- 3*median(Cret$vsample)
      if(length(degnames)>0){
       colnames(Cret$sample) <- c(colnamessample, degnames)
@@ -174,15 +234,33 @@ poscmp<-function(s,s2=NULL,rc=rep(FALSE,length=length(s2)),maxN=NULL,
       colnames(Cret$sample) <- colnamessample
      }
      Cret$sample[,"mem.optimism"] <- exp(Cret$sample[,"mem.optimism"])
+     #
+     # Transform natural CMP parametrization to mean
+     # parametrization (for the mean visibility value)
+     #
+     a <- Cret$sample[,c("mem.optimism","mem.scale")]
+     a[,1] <- a[,1]*mean(Cret$vsample)
+     a <- t(apply(a,1,cmp.to.mu.sd,K=K,force=TRUE,max.mu=K))
+     a[,1] <- Cret$sample[,"mem.optimism"]
+     colnames(a) <- c("mem.optimism","mem.sigma")
+     nas <- apply(a,1,function(x){any(is.na(x))})
+     if(!all(nas)){
+      inas <- sample(seq_along(nas)[!nas],size=sum(nas),replace=TRUE)
+      a[nas,] <- a[inas,]
+#     Cret$sample[,c("mem.optimism","mem.scale")] <- a
+      Cret$sample <- cbind(Cret$sample,a[,2])
+      colnames(Cret$sample)[ncol(Cret$sample)] <- "mem.sigma"
+     }
     }else{
-     colnamessample <- c("N","mu","sigma","degree1","totalsize")
-     max.mu <- 2*mean.prior.degree
+     colnamessample <- c("N","mu","sigma","visibility1","totalsize")
+     max.mu <- 2*mean.prior.visibility
      if(length(degnames)>0){
       colnames(Cret$sample) <- c(colnamessample, degnames)
      }else{
       colnames(Cret$sample) <- colnamessample
      }
     }
+   if(F){
     #
     # Transform observed mean parametrization to log-normal
     # parametrization
@@ -193,37 +271,39 @@ poscmp<-function(s,s2=NULL,rc=rep(FALSE,length=length(s2)),maxN=NULL,
     Cret$sample <- cbind(Cret$sample,Cret$sample[,c("mu","sigma")])
     colnames(Cret$sample)[ncol(Cret$sample)-(1:0)] <- c("lambda","nu")
     # Transform to mean value parametrization 
-    a <- t(apply(Cret$sample[,c("mu","sigma")],1,cmp.mu, max.mu=max.mu))
+    a <- t(apply(Cret$sample[,c("mu","sigma")],1,cmp.to.mu.sd, max.mu=max.mu))
     nas <- apply(a,1,function(x){any(is.na(x))})
     if(!all(nas)){
      inas <- sample(seq_along(nas)[!nas],size=sum(nas),replace=TRUE)
      a[nas,] <- a[inas,]
-#    Cret$sample[,c("mu","sigma")] <- t(apply(Cret$sample[,c("mu","sigma")],1,cmp.mu,max.mu=5*mean.prior.degree)))
+#    Cret$sample[,c("mu","sigma")] <- t(apply(Cret$sample[,c("mu","sigma")],1,cmp.to.mu.sd,max.mu=5*mean.prior.visibility)))
      Cret$sample[,c("mu","sigma")] <- a
     }else{
-      warning(paste("All the lambda and nu parameters are extreme. mean and sigma are on the natural scale."), call. = FALSE)
+      warning(paste("All the lambda and nu parameters are extreme. The mean and sigma are on the natural scale."), call. = FALSE)
     }
+   }
 #   # PLN mean
 #   Cret$sample <- cbind(Cret$sample,Cret$sample[,c("mem.optimism")])
-#   colnames(Cret$sample)[ncol(Cret$sample)] <- c("mem.degree.mean")
-#   mean.degree <- sum(seq(along=Cret$nk)*Cret$ppos)
-#   print(mean.degree)
-#   Cret$sample[,"mem.degree.mean"] <- exp(log(mean.degree)+Cret$sample[,"mem.optimism"]+0.5*Cret$sample[,"mem.sd"])
+#   colnames(Cret$sample)[ncol(Cret$sample)] <- c("mem.visibility.mean")
+#   mean.visibility <- sum(seq(along=Cret$nk)*Cret$ppos)
+#   print(mean.visibility)
+#   Cret$sample[,"mem.visibility.mean"] <- exp(log(mean.visibility)+Cret$sample[,"mem.optimism"]+0.5*Cret$sample[,"mem.scale"])
     #
 #   Cret$Nk<-Cret$nk/sum(Cret$nk)
-    Cret$predictive.degree.count<-Cret$nk / samplesize
+    Cret$predictive.visibility.count<-Cret$nk / samplesize
     Cret$nk<-NULL
-    Cret$predictive.degree<-Cret$ppos
+    Cret$predictive.visibility<-Cret$posu
+    Cret$predictive.degree<-Cret$posd
     Cret$ppos<-NULL
     endrun <- burnin+interval*(samplesize-1)
     attr(Cret$sample, "mcpar") <- c(burnin+1, endrun, interval)
     attr(Cret$sample, "class") <- "mcmc"
-    ### compute modes of posterior samples,Maximum A Posterior (MAP) values N, mu, sigma, degree1
+    ### compute modes of posterior samples,Maximum A Posterior (MAP) values N, mu, sigma, visibility1
     Cret$MAP <- apply(Cret$sample,2,mode.density)
     Cret$MAP["N"] <- mode.density(Cret$sample[,"N"],lbound=n,ubound=prior$maxN)
     if(!is.null(s2)){Cret$n <- Cret$n1 +  Cret$n2 - Cret$n0}
 #
-#   Cret$MSE <- c(((prior$x-mean.prior.degree)^2)*prior$lprior/sum(prior$lprior),mean((Cret$sample[,"N"]-mean.prior.degree)^2))
+#   Cret$MSE <- c(((prior$x-mean.prior.visibility)^2)*prior$lprior/sum(prior$lprior),mean((Cret$sample[,"N"]-mean.prior.visibility)^2))
     Cret$maxN <- prior$maxN
     Cret$quartiles.prior.size <- prior$quartiles.prior.size
     Cret$mode.prior.size <- prior$mode.prior.size
