@@ -9,46 +9,48 @@
 #include <math.h>
 
 void gcmpvis (int *pop,
-            int *nk, 
-            int *K, 
-            int *n, 
+            int *K,
+            int *n,
             int *samplesize, int *burnin, int *interval,
-            double *mu, double *dfmu, 
+            double *mu, double *dfmu,
             double *sigma, double *dfsigma,
-            double *beta0muprior, double *beta0sigmaprior, 
-            double *beta1muprior, double *beta1sigmaprior, 
-            double *memlnopt, double *memdflnopt,
+            double *lnlam, double *nu,
+            double *beta0muprior, double *beta0sigmaprior,
+            double *beta1muprior, double *beta1sigmaprior,
+            double *lmemmu, double *memdfmu,
             double *memnu, double *memdfnu,
             int *Npi,
-            int *srd, 
-            double *numrec, 
+            int *srd,
+            int *numrec,
             double *rectime,
             int *maxcoupons,
-            double *lnlamproposal, 
-            double *nuproposal, 
-            double *beta0proposal, double *beta1proposal, 
-            double *memlnoptproposal, double *memnuproposal,
-            int *N, int *maxN, 
-            double *sample, 
-            int *vsample, 
-            double *ppos, 
-            double *lpriorm, 
+            double *lnlamproposal,
+            double *nuproposal,
+            double *beta0proposal, double *beta1proposal,
+            double *lmemmuproposal, double *memnuproposal,
+            int *N, int *maxN,
+            double *sample,
+            int *vsample,
+            double *posu,
+            double *posd,
+            double *lpriorm,
             int *burnintheta,
             int *burninbeta,
             int *verbose
                          ) {
   int dimsample, Np;
   int step, staken, getone=1, intervalone=1, verboseMHcmp = 0;
-  int i, j, k, ni, Ni, Ki, isamp, iinterval, isamplesize, iburnin;
-  double lnlami, nui, dsamp;
-  double dmu, dsigma;
-  double dbeta0, dbeta0sd, dbeta1, dbeta1sd;
-  double dmemlnopt, dmemnu;
-  double beta0i, beta1i, memlnopti, memnui;
+  int i, ni, Ni, Ki, isamp, iinterval, isamplesize, iburnin;
+  int j, k;
+  int umax;
+  double mui, sigmai, lnlami, nui, dsamp;
+  double sigma2i;
+  double dbeta0, dbeta1;
+  double dlmemmu, dmemnu;
+  double beta0i, beta1i, lmemmui, memnui;
   int tU, sizei, imaxN, imaxm, give_log0=0, give_log1=1;
-  int maxpop;
   double r, gammart, pis, Nd;
-  double temp;
+  double temp, uprob;
   double rtprob, lliki;
   int maxc;
   double errval=0.0000000001, lzcmp;
@@ -64,14 +66,10 @@ void gcmpvis (int *pop,
   iinterval=(*interval);
   iburnin=(*burnin);
   Np=(*Npi);
-  dsigma=(*sigma);
   dbeta0=(*beta0muprior);
   dbeta1=(*beta1muprior);
-  dbeta0sd=(*beta0sigmaprior);
-  dbeta1sd=(*beta1sigmaprior);
-  dmemlnopt=(*memlnopt);
+  dlmemmu=(*lmemmu);
   dmemnu=(*memnu);
-  dmu=(*mu);
   maxc=(*maxcoupons);
   
   dimsample=5+Np+4;
@@ -79,39 +77,50 @@ void gcmpvis (int *pop,
   double *pi = (double *) malloc(sizeof(double) * Ki);
   double *pd = (double *) malloc(sizeof(double) * Ki);
   double *pd2 = (double *) malloc(sizeof(double) * Ki);
-  int *d = (int *) malloc(sizeof(int) * ni);
+  int *u = (int *) malloc(sizeof(int) * imaxN);
   int *b = (int *) malloc(sizeof(int) * ni);
   int *Nk = (int *) malloc(sizeof(int) * Ki);
+  int *nk = (int *) malloc(sizeof(int) * Ki);
   int *Nkpos = (int *) malloc(sizeof(int) * Ki);
   double *lpm = (double *) malloc(sizeof(double) * imaxm);
   double *pdegi = (double *) malloc(sizeof(double) * (Np+1));
   double *psample = (double *) malloc(sizeof(double) * (Np+1));
   double *lnlamsample = (double *) malloc(sizeof(double));
+  double *nusample = (double *) malloc(sizeof(double));
   double *beta0sample = (double *) malloc(sizeof(double));
   double *beta1sample = (double *) malloc(sizeof(double));
-  double *nusample = (double *) malloc(sizeof(double));
-  double *memlnoptsample = (double *) malloc(sizeof(double));
+  double *lmemmusample = (double *) malloc(sizeof(double));
   double *memnusample = (double *) malloc(sizeof(double));
 
-  maxpop=0;
-  for (i=0; i<ni; i++){
-    if((pop[i]>0) && (pop[i] <= Ki)){ d[i]=pop[i];}
-    if(pop[i]==0){ d[i]=1;}
-    if(pop[i]>Ki){ d[i]=Ki;}
-    if(pop[i]>maxpop){maxpop=pop[i];}
+  for (i=0; i<Ki; i++){
+    nk[i]=0;
   }
-  b[ni-1]=d[ni-1];
+  uprob=ni;
+  for (i=0; i<ni; i++){
+    if((pop[i]>0) && (pop[i] <= Ki)){ u[i]=pop[i];}
+    if( pop[i]==0){ u[i]=1;}
+    if( pop[i]>Ki){ u[i]=Ki; uprob--;}
+    nk[u[i]-1]=nk[u[i]-1]+1;
+  }
+  uprob/=ni;
+  uprob = 0.5 + uprob/2.0;
+// Rprintf("uprob %f\n", uprob);
+  b[ni-1]=u[ni-1];
   for (i=(ni-2); i>=0; i--){
-    b[i]=b[i+1]+d[i];
+    b[i]=b[i+1]+u[i];
   }
   for (i=0; i<Ki; i++){
      Nk[i]=nk[i];
      Nkpos[i]=0;
-     ppos[i]=0.;
+     posu[i]=0.;
+     posd[i]=0.;
+  }
+  for (i=ni; i<imaxN; i++){
+    u[i]=u[(int)trunc(10*unif_rand()+ni-10)];
   }
   tU=0;
   for (i=ni; i<Ni; i++){
-    tU+=pop[i];
+    tU+=u[i];
   }
   /* Draw initial phis */
   r=0.;
@@ -124,10 +133,10 @@ void gcmpvis (int *pop,
   }
   beta0sample[0] = dbeta0;
   beta1sample[0] = dbeta1;
-  memlnoptsample[0] = dmemlnopt;
+  lmemmusample[0] = dlmemmu;
   memnusample[0] = dmemnu;
-  lnlamsample[0] = dmu;
-  nusample[0] = dsigma;
+  lnlamsample[0] = (*lnlam);
+  nusample[0] = (*nu);
 
   isamp = 0;
   step = -iburnin;
@@ -147,7 +156,6 @@ void gcmpvis (int *pop,
      for (i=0; i<Np; i++){
       pdegi[i] = psample[i];
      }
-//if(nui > 4.0 || lnlami > 4.5) Rprintf("lnlami %f nui %f dfmu %f\n", lnlami, nui, (*dfmu));
     }
 
     /* Compute the unit distribution (given the new theta = (lnlam, nu)) */
@@ -155,15 +163,14 @@ void gcmpvis (int *pop,
     lzcmp = zcmp(exp(lnlami), nui, errval, Ki, give_log1);
     if(lzcmp < -100000.0){continue;}
     pi[Np]=cmp(Np+1,lnlami,nui,lzcmp,give_log0);
-//Rprintf("lnlami %f nui %f lzcmp %f pi %f\n", lnlami, nui, lzcmp, pi[Np]);
+    pis+=pi[Np];
     for (i=Np+1; i<Ki; i++){
       pi[i]=pi[i-1]*exp(lnlami-nui*log((double)(i+1)));
+      pis+=pi[i];
     }
-//  Rprintf("isamp %d pis %f\n", isamp, pis);
-    pis=1.-exp(-lzcmp);
+//  pis=1.-exp(-lzcmp);
     for (i=0; i<Ki; i++){
       pi[i]/=pis;
-//Rprintf("i %d pi %f pi0 %f\n", i, pi[i], pi0[i], pis, pis0);
     }
     pis=1.;
     for (i=0; i<Np; i++){
@@ -176,33 +183,45 @@ void gcmpvis (int *pop,
     for (i=0; i<Np; i++){
       pi[i]=pdegi[i];
     }
+    temp=0.;
+    for (umax=1; umax<=Ki; umax++){
+      temp+=pi[umax-1];
+      if(temp > uprob) break;
+    }
+
+    // Now computes mean and s.d. from log-lambda and nu
+    mui=0.0;
+    sigma2i=0.0;
+    for (i=0; i<Ki; i++){
+      mui+=pi[i]*(i+1);
+      sigma2i+=pi[i]*(i+1)*(i+1);
+    }
+    sigma2i=sigma2i-mui*mui;
+    sigmai = sqrt(sigma2i);
 
     /* Draw new beta using MCMC */
     if (step == -iburnin || step==(20*(step/20))) { 
-//  if (step == 3.141) { 
-     MHcmpmem(d,n,K,beta0muprior,beta0sigmaprior,beta1muprior,beta1sigmaprior,
-       memlnopt,memdflnopt,memnu,memdfnu,srd,numrec,rectime,maxcoupons,
+//  if (step == -iburnin) { 
+     MHcmpmem(u,n,K,beta0muprior,beta0sigmaprior,beta1muprior,beta1sigmaprior,
+       lmemmu,memdfmu,memnu,memdfnu,srd,numrec,rectime,maxcoupons,
        beta0proposal,beta1proposal,
-       memlnoptproposal,memnuproposal,
-       beta0sample, beta1sample,memlnoptsample,memnusample,
+       lmemmuproposal,memnuproposal,
+       beta0sample, beta1sample,lmemmusample,memnusample,
        &getone, &staken, burninbeta, &intervalone, 
        &verboseMHcmp);
      beta0i=beta0sample[0];
      beta1i=beta1sample[0];
-     memlnopti=memlnoptsample[0];
+     lmemmui=lmemmusample[0];
      memnui=memnusample[0];
-//Rprintf("memlnopti: %f memnui %f beta0i %f beta1i %f\n",memlnopti,memnui,beta0i,beta1i);
+//  Rprintf("step %d lmemmui: %f rmemnui %f beta0i %f beta1i %f\n",step,lmemmui,rmemnui,beta0i,beta1i);
 
-//  if(i== -59){
-    /* Draw true degrees (sizes) based on the reported degrees*/
+    /* Draw true unit sizes based on the reported degrees*/
     // First reset counts
-//Rprintf("nk[i]: ");
     for (i=0; i<Ki; i++){
-//Rprintf("%d ", nk[i]);
      nk[i]=0;
     }
-//Rprintf("\n");
     for (j=0; j<ni; j++){
+      if(srd[j] <= Ki){
       temp = beta0i + beta1i*rectime[j];
       rtprob = exp(temp)/(1.0+exp(temp));
 //    Multiply by the PoissonLogNormal PMF for observation
@@ -217,44 +236,65 @@ void gcmpvis (int *pop,
           lliki += log(1.0-pbinom(maxc-1.0,(i+1),rtprob,give_log0,give_log0));
         }
         if(srd[j]>=0){
-//        lliki += log(poilog(srd[j],log((double)(i+1))-memlnopti,memnui));
 //        Use CMP localized
-          temp=memlnopti + log((double)(i+1));
+          temp=lmemmui + log((double)(i+1));
           pis=0.;
           lzcmp = zcmp(exp(temp), memnui, errval, Ki, give_log1);
           if(lzcmp < -100000.0){Rprintf("badlzcmp ");continue;}
           pd2[0]=cmp(1,temp,memnui,lzcmp,give_log0);
+          pis+=pd2[0];
           for (k=1; k<Ki; k++){
             pd2[k]=pd2[k-1]*exp(temp-memnui*log((double)(k+1)));
+            pis+=pd2[k];
           }
-          pis=1.-exp(-lzcmp);
-//if(j==6) Rprintf("pd2[i]: %f ",pis);
+//        pis=1.-exp(-lzcmp);
           for (k=0; k<Ki; k++){
             pd2[k]/=pis;
-//if(j==6) Rprintf("%f ", pd2[i]);
           }
-//if(j==6) Rprintf("\n");
-//if(j==6) Rprintf("pd2[srd]: %f\n",pd2[srd[j]-1]);
-          lliki += log(pd2[srd[j]-1]);
+          if(srd[j] <= Ki){
+            lliki += log(pd2[srd[j]-1]);
+//        }else{
+/////       lliki += log(pd2[Ki-1]);
+//	    if(temp<1.0){
+//              lliki += log(1.-temp);
+//	    }else{
+//              lliki += -100.0;
+//	    }
+// Rprintf("lliki0 lmemmui %f memnui %f srd: %d llik %f p[50] %f p %f\n",lmemmui,memnui,srd[i],lliki,pd2[49],(1.-temp));
+// if((1.-temp) > 0.99){
+//        for (k=0; k<Ki; k++){ Rprintf("k %d pd2[k] %f\n",k,pd2[k]);}
+// }
+          }
         }
         pd[i]=pi[i]*exp(lliki);
        }else{
         pd[i]=0.0;
        }
+//    if((i+1) != d[j]){
+//      pd[i] = 0.0;
+//    }
 //     pd[i]=pi[i];
 //Rprintf("srd[j] %d i %d pd[i] %f\n", srd[j],i, pd[i]);
 //Rprintf("srd[j] %d numrec %f i %d lliki %f\n", srd[j], numrec[j], i+1, lliki);
       }
       // Set up pd to be cumulative for the random draws
 //if(j==6) Rprintf("pd[i]: ");
+    pis=0.;
+    for (i=0; i<Ki; i++){
+      pis+=pd[i];
+    }
+    for (i=0; i<Ki; i++){
+      pd[i]/=pis;
+    }
+//Rprintf("j %d d[j] %d pd: %f %f %f %f\n",j, d[j], pd[d[j]-2], pd[d[j]-1], pd[d[j]], pd[d[j]+1]);
       for (i=1; i<Ki; i++){
 //if(j==6) Rprintf("%f ", pd[i]);
        pd[i]=pd[i-1]+pd[i];
       }
 //if(j==6) Rprintf("\n");
-//if(j==6)Rprintf("beta0i %f beta1i %f memlnopti %f memnui %f rtprob %f pd[Ki-1] %f\n", beta0i,beta1i,memlnopti,memnui,rtprob,pd[Ki-1]);
+//if(j==6)Rprintf("beta0i %f beta1i %f lmemmui %f rmemnui %f rtprob %f pd[Ki-1] %f\n", beta0i,beta1i,lmemmui,rmemnui,rtprob,pd[Ki-1]);
       if(pd[Ki-1]<0.00000000001){
-   Rprintf("fixed bad pd[Ki-1] %f\n", pd[Ki-1]);
+       Rprintf("fixed bad pd[Ki-1] %f\n", pd[Ki-1]);
        for (i=0; i<Ki; i++){
         pd[i]=pi[i];
        }
@@ -269,20 +309,42 @@ void gcmpvis (int *pop,
       for (sizei=1; sizei<=Ki; sizei++){
         if(temp <= pd[sizei-1]) break;
       }
+      }else{
+      /* Deal with the outliers */
+        sizei=umax;
+      }
+// Next line to force unit sizes to degrees
+// sizei=u[j];
+
       nk[sizei-1]=nk[sizei-1]+1;
-      d[j]=sizei;
-if(d[j] < numrec[j]) Rprintf("Warning: j %d d[j] %d numrec[j] %d maxc %d: %f %f %f %f\n",j,d[j],numrec[j],maxc,pd[0],pd[1],pd[2],pd[3]);
+      u[j]=sizei;
+
+//    if(u[j] < numrec[j]) Rprintf("Warning: j %d u[j] %d numrec[j] %d maxc %d: %f %f %f %f\n",j,u[j],numrec[j],maxc,pd[0],pd[1],pd[2],pd[3]);
 //Rprintf("j %d dis %d sizei %d pd[Ki-1] %f\n", j, ddis, sizei, pd[Ki-1]);
      } 
      // Rebuild b
-     b[ni-1]=d[ni-1];
+     b[ni-1]=u[ni-1];
      for (i=(ni-2); i>=0; i--){
-      b[i]=b[i+1]+d[i];
+      b[i]=b[i+1]+u[i];
      }
-// Rprintf("j %d d[j] %d pd[Ki-1] %f\n", j, d[j], pd[Ki-1]);
+// Rprintf("j %d u[j] %d pd[Ki-1] %f\n", j, u[j], pd[Ki-1]);
+//
      /* End of imputed unit sizes for observed */
     }
+
 //  }
+// Rprintf("step %d b[ni-2] %d; End of imputed unit sizes for observed\n", step, b[ni-2]);
+
+    /* Draw phis */
+    tU=0;
+    for (i=ni; i<Ni; i++){
+      tU+=u[i];
+    }
+    r=0.;
+    for (i=0; i<ni; i++){
+//    phi[i]=(tU+b[i])*exp_rand();
+      r+=exp_rand()/(tU+b[i]);
+    }
 
     /* Draw new N */
 
@@ -314,17 +376,6 @@ if(d[j] < numrec[j]) Rprintf("Warning: j %d d[j] %d numrec[j] %d maxc %d: %f %f
     Ni += ni;
     if(Ni > imaxN) Ni = imaxN;
 
-    /* Draw phis */
-    tU=0;
-    for (i=ni; i<Ni; i++){
-      tU+=pop[i];
-    }
-    r=0.;
-    for (i=0; i<ni; i++){
-//    phi[i]=(tU+b[i])*exp_rand();
-      r+=exp_rand()/(tU+b[i]);
-    }
-
     /* Draw unseen sizes */
     for (i=0; i<Ki; i++){
 //Rprintf("i %d nk[i] %f\n", i, nk[i]/(1.0*ni));
@@ -354,7 +405,7 @@ if(d[j] < numrec[j]) Rprintf("Warning: j %d d[j] %d numrec[j] %d maxc %d: %f %f
        }
       }
 //    if(sizei >= Ki){sizei=Ki-1;}
-      pop[i]=sizei;
+      u[i]=sizei;
 //    if((sizei <= 0) | (sizei > Ki-1)) Rprintf("sizei %d r %f\n", sizei,r);
       Nk[sizei-1]=Nk[sizei-1]+1;
     }
@@ -363,9 +414,9 @@ if(d[j] < numrec[j]) Rprintf("Warning: j %d d[j] %d numrec[j] %d maxc %d: %f %f
       Nd=(double)Ni;
       sample[isamp*dimsample  ]=Nd;
 //if(nui > 4.0 || lnlami > 4.5) Rprintf("sample: %f %f\n", lnlami,nui);
-// Rprintf("sample: %f %f\n", lnlami,nui);
-      sample[isamp*dimsample+1]=lnlami;
-      sample[isamp*dimsample+2]=nui;
+// Rprintf("sample: step %d %f %f %d %d %d\n", step, lnlami,nui,Np,isamp*dimsample+8,isamp*ni+ni-1);
+      sample[isamp*dimsample+1]=mui;
+      sample[isamp*dimsample+2]=sigmai;
       sample[isamp*dimsample+3]=(double)(Nk[0]);
       temp=0.0;
       for (i=0; i<Ki; i++){
@@ -374,17 +425,35 @@ if(d[j] < numrec[j]) Rprintf("Warning: j %d d[j] %d numrec[j] %d maxc %d: %f %f
       sample[isamp*dimsample+4]=temp;
       sample[isamp*dimsample+5]=beta0i;
       sample[isamp*dimsample+6]=beta1i;
-      sample[isamp*dimsample+7]=memlnopti;
+      sample[isamp*dimsample+7]=lmemmui;
       sample[isamp*dimsample+8]=memnui;
       for (i=0; i<Np; i++){
         sample[isamp*dimsample+9+i]=pdegi[i];
       }
       for (i=0; i<Ki; i++){
         Nkpos[i]=Nkpos[i]+Nk[i];
-        ppos[i]+=((Nk[i]*1.)/Nd);
+        posu[i]+=((Nk[i]*1.)/Nd);
       }
       for (i=0; i<ni; i++){
-        vsample[isamp*ni+i]=d[i];
+        vsample[isamp*ni+i]=u[i];
+        if((srd[i]>0) && (srd[i] <= Ki)){ posd[srd[i]-1]+=(1./Nd); }
+      }
+      for (i=ni; i<Ni; i++){
+        temp=lmemmui + log((double)(u[i])+1.);
+        pis=0.;
+        lzcmp = zcmp(exp(temp), memnui, errval, Ki, give_log1);
+        if(lzcmp < -100000.0){Rprintf("badlzcmp ");continue;}
+        pd[0]=cmp(1,temp,memnui,lzcmp,give_log0);
+        pis+=pd[0];
+        for (k=1; k<Ki; k++){
+          pd[k]=pd[k-1]*exp(temp-memnui*log((double)(k+1)));
+          pis+=pd[k];
+        }
+//      pis=1.-exp(-lzcmp);
+        for (k=0; k<Ki; k++){
+          pd[k]/=pis;
+          posd[k]+=(pd[k]/Nd);
+        }
       }
       isamp++;
       if (*verbose && isamplesize==(isamp*(isamplesize/isamp))) Rprintf("Taken %d samples...\n", isamp);
@@ -395,17 +464,20 @@ if(d[j] < numrec[j]) Rprintf("Warning: j %d d[j] %d numrec[j] %d maxc %d: %f %f
   dsamp=((double)isamp);
   for (i=0; i<Ki; i++){
     nk[i]=Nkpos[i];
-    ppos[i]/=dsamp;
+    posu[i]/=dsamp;
+    posd[i]/=dsamp;
   }
   for (i=0; i<ni; i++){
-     pop[i]=d[i];
+     pop[i]=u[i];
   }
+ // Rprintf("ni %d Ki %d\n", ni, Ki);
   PutRNGstate();  /* Disable RNG before returning */
   free(pi);
   free(pd);
   free(pd2);
-  free(d);
+  free(u);
   free(b);
+  free(nk);
   free(Nk);
   free(Nkpos);
   free(lpm);
@@ -415,22 +487,22 @@ if(d[j] < numrec[j]) Rprintf("Warning: j %d d[j] %d numrec[j] %d maxc %d: %f %f
   free(nusample);
   free(beta0sample);
   free(beta1sample);
-  free(memlnoptsample);
+  free(lmemmusample);
   free(memnusample);
 }
 
-void MHcmpmem (int *d, int *n, int *K,
+void MHcmpmem (int *u, int *n, int *K,
             double *beta0, double *beta0sd, double *beta1, double *beta1sd, 
-            double *memlnopt, double *memdflnopt,
+            double *lmemmu, double *memdfmu,
             double *memnu, double *memdfnu,
             int *srd, 
-            double *numrec, 
+            int *numrec, 
             double *rectime,
             int *maxcoupons,
             double *beta0proposal, double *beta1proposal, 
-            double *memlnoptproposal, double *memnuproposal, 
+            double *lmemmuproposal, double *memnuproposal, 
             double *beta0sample, double *beta1sample,
-            double *memlnoptsample, double *memnusample,
+            double *lmemmusample, double *memnusample,
             int *samplesize, int *staken, int *burnin, int *interval,
             int *verbose
          ) {
@@ -441,22 +513,20 @@ void MHcmpmem (int *d, int *n, int *K,
   double temp, rtprob;
   double beta0star, beta1star, beta0i, beta1i;
   double qi, qstar, lliki, llikstar;
-  double memlnoptstar, memnustar, memlnopti, memnui;
-  double memnu2i, memnu2star;
+  double lmemmustar, memnustar, lmemmui, memnui;
+  double rmemnui, rmemnustar;
   double pibeta0star, pibeta0i;
-  double pibeta1star, pibeta1i;
+  double pibeta1star=0.0, pibeta1i=0.0;
   double pimemstar, pimemi;
   double dbeta0, dbeta0sd, dbeta1, dbeta1sd;
-  double dmemlnopt, dmemdflnopt, dmemdfnu, rmemdflnopt;
-  double dmemnu, dmemnu2;
+  double dlmemmu, dmemdfmu, dmemdfnu, rmemdfmu;
+  double dmemnu, dmemnur;
   double dbeta0proposal, dbeta1proposal;
-  double dmemlnoptproposal, dmemnuproposal;
+  double dlmemmuproposal, dmemnuproposal;
   double pis, errval=0.0000000001, lzcmp;
 
   Ki=(*K);
   double *pd = (double *) malloc(sizeof(double) * Ki);
-//double *pi = (double *) malloc(sizeof(double) * Ki);
-//double *pstar = (double *) malloc(sizeof(double) * Ki);
 
 //GetRNGstate();  /* R function enabling uniform RNG */
 
@@ -467,15 +537,15 @@ void MHcmpmem (int *d, int *n, int *K,
   dbeta0sd=(*beta0sd);
   dbeta1=(*beta1);
   dbeta1sd=(*beta1sd);
-  dmemlnopt=(*memlnopt);
+  dlmemmu=(*lmemmu);
   dmemnu=(*memnu);
-  dmemnu2=dmemnu*dmemnu;
-  dmemdflnopt=(*memdflnopt);
-  rmemdflnopt=sqrt(dmemdflnopt);
+  dmemnur=sqrt(dmemnu);
+  dmemdfmu=(*memdfmu);
+  rmemdfmu=sqrt(dmemdfmu);
   dmemdfnu=(*memdfnu);
   dbeta0proposal=(*beta0proposal);
   dbeta1proposal=(*beta1proposal);
-  dmemlnoptproposal=(*memlnoptproposal);
+  dlmemmuproposal=(*lmemmuproposal);
   dmemnuproposal=(*memnuproposal);
 
   // First set starting values
@@ -485,54 +555,69 @@ void MHcmpmem (int *d, int *n, int *K,
   maxc=(*maxcoupons);
   beta0i = beta0sample[0];
   beta1i = beta1sample[0];
-  memlnopti = memlnoptsample[0];
+  lmemmui = lmemmusample[0];
   memnui = memnusample[0];
-  memnu2i = memnui*memnui;
+  rmemnui = sqrt(memnui);
 
   // Compute initial current lik
   lliki = 0.0;
   for (i=0; i<ni; i++){
     temp = beta0i + beta1i*rectime[i];
     rtprob = exp(temp)/(1.0+exp(temp));
-//  if((numrec[i] <= d[i]) && ((maxc-1) <= d[i])){
-     if((d[i] <= maxc)|(numrec[i]<maxc)){
-      lliki += dbinom(numrec[i],d[i],rtprob,give_log1);
+//  if((numrec[i] <= u[i]) && ((maxc-1) <= u[i])){
+    if(numrec[i] <= u[i]){
+     if((u[i] <= maxc)|(numrec[i]<maxc)){
+       lliki += dbinom(numrec[i],u[i],rtprob,give_log1);
      }else{
-      lliki += log(1.0-pbinom(maxc-1.0,d[i],rtprob,give_log0,give_log0));
+       lliki += log(1.0-pbinom(maxc-1.0,u[i],rtprob,give_log0,give_log0));
      }
      if(srd[i]>=0){
-//    lliki += log(poilog(srd[i],log((double)(d[i]))-memlnopti,memnui));
+//    lliki += log(poilog(srd[i],log((double)(u[i]))-lmemmui,memnui));
 //    Use CMP localized
-      temp=memlnopti+log(d[i]);
+      temp=lmemmui+log((double)(u[i]));
       pis=0.;
       lzcmp = zcmp(exp(temp), memnui, errval, Ki, give_log1);
       if(lzcmp < -100000.0){Rprintf("badlzcmp ");continue;}
       pd[0]=cmp(1,temp,memnui,lzcmp,give_log0);
+      pis+=pd[0];
       for (k=1; k<Ki; k++){
         pd[k]=pd[k-1]*exp(temp-memnui*log((double)(k+1)));
+        pis+=pd[k];
       }
-      pis=1.-exp(-lzcmp);
+//    pis=1.-exp(-lzcmp);
       for (k=0; k<Ki; k++){
         pd[k]/=pis;
       }
-      lliki += log(pd[srd[i]-1]);
+      if(srd[i] <= Ki){
+        lliki += log(pd[srd[i]-1]);
+//    }else{
+///     lliki += log(pd[Ki-1]);
+//if(temp<1.0){
+//        lliki += log(1.-temp);
+//}else{
+//        lliki += -100.0;
+//}
+// Rprintf("lliki srd: %d llik %f p[50] %f p %f\n",srd[i],lliki,pd[49],(1.-temp));
+      }
      }
-//  }else{
-//   lliki = -100000.0; 
-//  }
+     // Next 2 lines possibly comment out
+    }else{
+     lliki = -100000.0; 
+    }
   }
   if(!isfinite(lliki)) lliki = -100000.0; 
 
-//Rprintf("New call: lliki=%f memlnopti=%f memnui=%f rtprob=%f\n",lliki,memlnopti,memnui,rtprob);
+//Rprintf("New call: lliki=%f lmemmui=%f memnui=%f rtprob=%f\n",lliki,lmemmui,memnui,rtprob);
 
   // Compute initial prior
   pibeta0i = dnorm(beta0i, dbeta0, dbeta0sd, give_log1);
   if(dbeta1sd > 0.0) pibeta1i = dnorm(beta1i, dbeta1, dbeta1sd, give_log1);
-  pimemi = dnorm(memlnopti, dmemlnopt, memnui/rmemdflnopt, give_log1);
-  pimemi = pimemi+dsclinvchisq(memnu2i, dmemdfnu, dmemnu2);
+  pimemi = dnorm(lmemmui, dlmemmu, rmemnui/rmemdfmu, give_log1);
+  pimemi = pimemi+dsclinvchisq(memnui, dmemdfnu, dmemnu);
+//pimemi = dsclinvchisq(memnui, dmemdfnu, dmemnu);
 
-//Rprintf("dmemdflnopt %f dmemdfnu %f ddflnopt %f ddfsigma %f\n", memdflnopt, memdfnu, dflnopt, dfsigma);
-//Rprintf("dmemdflnopt %f dmemdfnu %f rmemdflnopt %f\n", dmemdflnopt, dmemdfnu, rmemdflnopt);
+//Rprintf("dmemdfmu %f dmemdfnu %f ddfmu %f ddfsigma %f\n", memdfmu, memdfnu, dfmu, dfsigma);
+//Rprintf("dmemdfmu %f dmemdfnu %f rmemdfmu %f\n", dmemdfmu, dmemdfnu, rmemdfmu);
 
   // Now do the MCMC updates (starting with the burnin updates)
   while (isamp < isamplesize && step < iburnin) {
@@ -543,56 +628,74 @@ void MHcmpmem (int *d, int *n, int *K,
     }else{
       beta1star = beta1i;
     }
-    /* Propose new memnu and memlnopt */
-    memlnoptstar = rnorm(memlnopti, dmemlnoptproposal);
-//  memnustar = rnorm(memnui, dmemnuproposal);
-    memnu2star = memnu2i*exp(rnorm(0., dmemnuproposal));
-    memnustar = sqrt(memnu2star);
+    /* Propose new memnu and lmemmu */
+    lmemmustar = rnorm(lmemmui, dlmemmuproposal);
+//  lmemmustar = lmemmui;
+//  lmemmustar = 0.0;
+//  rmemnustar = rnorm(memnui, dmemnuproposal);
+    memnustar = memnui*exp(rnorm(0., dmemnuproposal));
+    rmemnustar = sqrt(memnustar);
 
 // for (i=0; i<ni; i++){
-//    Rprintf("%f %i %f\n",numrec[i],d[i],rectime[i]);
+//    Rprintf("%f %i %f\n",numrec[i],u[i],rectime[i]);
 // }
   
     llikstar = 0.0;
     for (i=0; i<ni; i++){
       temp = beta0star + beta1star*rectime[i];
       rtprob = exp(temp)/(1.0+exp(temp));
-//    Rprintf("%f %f %f %f\n",llikstar,memlnoptstar,memnustar,rtprob);
-//    if((numrec[i] <= d[i]) && ((maxc-1) <= d[i])){
-       if((d[i] <= maxc)|(numrec[i]<maxc)){
-        llikstar += dbinom(numrec[i],d[i],rtprob,give_log1);
+//    Rprintf("%f %f %f %f\n",llikstar,lmemmustar,memnustar,rtprob);
+//    if((numrec[i] <= u[i]) && ((maxc-1) <= u[i])){
+      if(numrec[i] <= u[i]){
+       if((u[i] <= maxc)|(numrec[i]<maxc)){
+        llikstar += dbinom(numrec[i],u[i],rtprob,give_log1);
 //      Rprintf("< %f %f %f %f\n",numrec[i],rtprob[i],u,dbinom(numrec[i],u,rtprob[i],give_log0));
        }else{
-        llikstar += log(1.0-pbinom(maxc-1.0,d[i],rtprob,give_log0,give_log0));
+        llikstar += log(1.0-pbinom(maxc-1.0,u[i],rtprob,give_log0,give_log0));
 //      Rprintf("< %f\n",1.0-pbinom(maxc-1.0,u,rtprob[i],give_log0,give_log0));
        }
        if(srd[i]>=0){
-//      llikstar += log(poilog(srd[i],log(d[i])-memlnoptstar,exp(memnustar)));
-//      Use CMP localized
-        temp=memlnoptstar+log(d[i]);
+//      llikstar += log(poilog(srd[i],log(u[i])-lmemmustar,exp(memnustar)));
+//      Use GP localized
+//      llans <- -x*lambda - theta + (x-1) * log(theta + x*lambda) +
+//                 log(theta) - lgamma(x+1)
+        temp=lmemmustar+log((double)(u[i]));
         pis=0.;
         lzcmp = zcmp(exp(temp), memnustar, errval, Ki, give_log1);
         if(lzcmp < -100000.0){Rprintf("badlzcmp ");continue;}
         pd[0]=cmp(1,temp,memnustar,lzcmp,give_log0);
+        pis+=pd[0];
         for (k=1; k<Ki; k++){
           pd[k]=pd[k-1]*exp(temp-memnustar*log((double)(k+1)));
+          pis+=pd[k];
         }
-        pis=1.-exp(-lzcmp);
+//      pis=1.-exp(-lzcmp);
         for (k=0; k<Ki; k++){
           pd[k]/=pis;
         }
-        llikstar += log(pd[srd[i]-1]);
-//  Rprintf("llikstar: %i %f %f %f %f %f %f %f\n",i, llikstar,memlnoptstar,memnustar,beta0star,beta1star,temp,rtprob);
+        if(srd[i] <= Ki){
+          llikstar += log(pd[srd[i]-1]);
+//      }else{
+////      llikstar += log(pd[Ki-1]);
+//if(temp<1.0){
+//         llikstar += log(1.-temp);
+//}else{
+//         llikstar += -100.0;
+//}
+// Rprintf("llikstar srd: %d llik %f p[50] %f p %f\n",srd[i],lliki,pd[49],(1.-temp));
+        }
+//  Rprintf("llikstar: %i %f %f %f %f %f %f %f\n",i, llikstar,lmemmustar,memnustar,beta0star,beta1star,temp,rtprob);
        }
-//    }else{
-//     llikstar = -100000.0; 
-//    }
+     // Next 2 lines possibly comment out
+      }else{
+       llikstar = -100000.0; 
+      }
     }
     if(!isfinite(llikstar)) llikstar = -100000.0; 
 
-//  Rprintf("llikstar: %f %f %f %f\n",llikstar,memlnoptstar,memnustar,rtprob);
+//  Rprintf("llikstar: %f %f %f %f\n",llikstar,lmemmustar,memnustar,rtprob);
 
-//    Rprintf("dn %f\n", poilog(srd[0],log(d[0])-memlnoptstar,exp(memnustar)));
+//    Rprintf("dn %f\n", poilog(srd[0],log(u[0])-lmemmustar,exp(memnustar)));
 
 //      Rprintf("dn %f\n", log(temp));
 //  if(!isnan(temp)) llikstar += temp; 
@@ -600,8 +703,9 @@ void MHcmpmem (int *d, int *n, int *K,
     /* Calculate pieces of the prior. */
     pibeta0star = dnorm(beta0star, dbeta0, dbeta0sd, give_log1);
     if(dbeta1sd > 0.0) pibeta1star = dnorm(beta1star, dbeta1, dbeta1sd, give_log1);
-    pimemstar = dnorm(memlnoptstar, dmemlnopt, memnustar/rmemdflnopt, give_log1);
-    pimemstar = pimemstar+dsclinvchisq(memnu2star, dmemdfnu, dmemnu2);
+    pimemstar = dnorm(lmemmustar, dlmemmu, rmemnustar/rmemdfmu, give_log1);
+    pimemstar = pimemstar+dsclinvchisq(memnustar, dmemdfnu, dmemnu);
+//  pimemstar = dsclinvchisq(memnustar, dmemdfnu, dmemnu);
 
     qi = dnorm(log(memnui/memnustar)/dmemnuproposal,0.,1.,give_log1)
          -log(dmemnuproposal*memnui);
@@ -636,9 +740,9 @@ void MHcmpmem (int *d, int *n, int *K,
       /* Make proposed changes */
       beta0i = beta0star;
       beta1i = beta1star;
-      memlnopti = memlnoptstar;
+      lmemmui = lmemmustar;
       memnui = memnustar;
-      memnu2i = memnu2star;
+      rmemnui = rmemnustar;
       lliki = llikstar;
       qi = qstar;
       pibeta0i = pibeta0star;
@@ -649,7 +753,7 @@ void MHcmpmem (int *d, int *n, int *K,
         /* record statistics for posterity */
         beta0sample[isamp]=beta0i;
         beta1sample[isamp]=beta1i;
-        memlnoptsample[isamp]=memlnopti;
+        lmemmusample[isamp]=lmemmui;
         memnusample[isamp]=memnui;
         isamp++;
         if (*verbose && isamplesize==(isamp*(isamplesize/isamp))) Rprintf("Taken %d MH samples...\n", isamp);
